/**
* This sample shows how to use an analog input sequence acquisition using DMA_SINGLE mode.
**/
#include "stdafx.h"

/**
* Pointer that will be allocated and that will enables to read the data of the board
**/
USHORT * data = NULL;
DWORD	dataSize = 0;

/**
* Interrupt routine that enables to get the data read by the board
**/
static void SequenceInterruptRoutine (HANDLE	deviceHandle,
							   BYTE		fifoOverflow,
							   DWORD	interruptSource,
							   DWORD	interruptMask,
							   PWORD	args,
							   DWORD	argsCount )
{
	UNREFERENCED_PARAMETER(args);
	/* switch through the possible interrupt sources */

	/* check the interrupt fifo overflow */
	if ( fifoOverflow )
	{
		printf("Error: interrupt fifo overflow. That means that your software do not handle the interrupts fast enough.\n");
	}

	/* DMA */
	if ( interruptSource & 0x4 )
	{
		if ( interruptMask & 0x4 )
		{
			printf("internal sequence fifo overflow \n");
		}
		else
		{
			LONG read = 0;
			/* get the sequence data */
			read = l_PCI3116_ReadSequenceData( deviceHandle, dataSize, data );
			if ( read < 0 )
			{
				printf("error %d in l_PCI3116_ReadSequenceData \n", (int) read);
			}
			else
			{
				DWORD i = 0;
				/* display the values */
				for( i = 0 ; i < argsCount ; i++ )
				{
					/* each 4 values, go to the line */
					if ( i % 4 == 0 )
					{
						printf("\n");
					}
					printf("0x%.4x ", data[i] );
				}
				printf("\n");
			}
		}
	}

	/* DMA counter overflow */
	if ( interruptSource & 0x8 )
	{
		printf("Error: DMA counter overflow. This stop the acquisition.\n");
	}

	/* PLD Fifo overflow */
	if ( interruptSource & 0x10 )
	{
		printf("Error: PLD Fifo overflow. This stop the acquisition.\n");
	}
}

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE boardHandle = NULL;
	ULONG readValue = 0;
	BYTE channels[16] = {0};
	BYTE gain[16] = {0};
	BYTE polarity[16] = {0};
	BYTE inputMode = 0;
	ULONG convertingTime = 0;
	ULONG sequenceCount = 0;
	DWORD sequenceBeforeInterrupt = 0;

	/* initialize the 4 channels of the acquisition */
	channels[0] = 0;
	channels[1] = 1;
	channels[2] = 2;
	channels[3] = 3;

	gain[0] = PCI3116_GAIN_1;
	gain[1] = PCI3116_GAIN_1;
	gain[2] = PCI3116_GAIN_1;
	gain[3] = PCI3116_GAIN_1;

	polarity[0] = PCI3116_OUTPUT_MODE_BIPOLAR;
	polarity[1] = PCI3116_OUTPUT_MODE_BIPOLAR;
	polarity[2] = PCI3116_OUTPUT_MODE_BIPOLAR;
	polarity[3] = PCI3116_OUTPUT_MODE_BIPOLAR;

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* This sample will initialize an acquisition using 4 channels and no delay. \n");
	printf("* The sequence is : \n");
	printf("*   - Channel 0, bipolar, gain 1 \n");
	printf("*   - Channel 1, bipolar, gain 1 \n");
	printf("*   - Channel 2, bipolar, gain 1 \n");
	printf("*   - Channel 3, bipolar, gain 1 \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* ask the input mode */
	ReadUlong("Select the input mode (0: Single-ended, 1: Differential) ", 0, 1, &readValue);
	switch ( readValue )
	{
		case 0:
			inputMode = PCI3116_INPUT_MODE_SINGLE_ENDED;
			break;
		case 1:
			inputMode = PCI3116_INPUT_MODE_DIFFERENTIAL;
			break;
	}

	/* set the input mode */
	returnValue = i_PCI3116_SetAnalogInputMode( boardHandle, inputMode );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3116_SetAnalogInputMode\n", returnValue);
		i_PCI3116_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* set the board interrupt routine */
	returnValue = i_PCI3116_SetBoardIntRoutine( boardHandle, SequenceInterruptRoutine );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3116_SetBoardIntRoutine \n", returnValue);
		i_PCI3116_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* ask the number of sequences to acquire */
	ReadUlong("How many sequences do you want to acquire (0: continuous) ? ", 0, 0xFFFFFFFF, &sequenceCount );

	/* ask the number of sequences before interrupt */
	ReadUlong("How many sequences do you want before the board generate an interrupt ? ", 1, 0xFFFFFFFF, &sequenceBeforeInterrupt );

	/* allocate memory */
	dataSize = sequenceBeforeInterrupt * 4 * sizeof(USHORT);
	data = (USHORT*)malloc( dataSize );
	if ( data == NULL )
	{
		printf("Error in malloc. data is null \n" );
		i_PCI3116_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* ask the converting time */
	ReadUlong("What converting time do you want to use? (5 to 65535000 - us) ", 5, 65535000, &convertingTime );
	
	printf("While the acquisition is running, press any key to stop the sample \n");
	printf("Press a key to start the acquisition \n");
	_getch();

	/* start the analog input sequence acquisition */
	returnValue = i_PCI3116_InitAndStartAnalogInputSequence(boardHandle,
		sequenceCount,					/* number of sequences to acquire (0 for continuous)*/
		sequenceBeforeInterrupt,		/* number of sequences in one interrupt */
		4,								/* 4 channels */
		channels,
		gain,
		polarity,
		convertingTime,
		PCI3116_NO_DELAY,				/* no delay */
		PCI3116_SOFT_TRIGGER_DISABLED,	/* no software trigger */
		PCI3116_EXT_TRIGGER_DISABLED,	/* no hardware trigger */
		0);

	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3116_StartAnalogInputSequence \n", returnValue);
		i_PCI3116_ResetBoardIntRoutine(boardHandle);
		i_PCI3116_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* 
	* endless loop, waiting an input from the user 
	* See the InterruptRoutine in the beginning of this file to see how the messages are displayed
	*/
	for(;;)
	{
		/* use hit keyboard ?*/
		if (_kbhit())
		{
			break;
		}
		Sleep(100);
	}
	
	/* stop the acquisition */
	returnValue = i_PCI3116_StopAnalogAcquisition( boardHandle );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3116_StopAnalogAcquisition \n", returnValue);
	}
	else
	{
		printf("i_PCI3116_StopAnalogAcquisition OK \n");
	}

	/* reset the interrupt routine */
	returnValue = i_PCI3116_ResetBoardIntRoutine( boardHandle );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3116_ResetBoardIntRoutine \n", returnValue);
	}
	else
	{
		printf("i_PCI3116_ResetBoardIntRoutine OK \n");
	}

	/* close the board */
	returnValue = i_PCI3116_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3116_CloseBoard \n", returnValue);
	}
	else
	{
		printf("i_PCI3116_CloseBoard OK \n");
	}

	/* free memory */
	free(data);

	/* exit */
	Sleep(2000);

	return 0;
}

