using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ZedGraph;
using System.Runtime.InteropServices;

namespace AutoRefresh
{
    public partial class AutoRefresh : Form
    {
        /// <summary>
        /// Handle to the first board
        /// </summary>
        private PCI3116_NET.PCI3116 board;

        /// <summary>
        /// Useful variable that enables us to make easy access on the channels boxes
        /// </summary>
        private CheckBox[] channelsBox;

        /// <summary>
        /// List of selected channels
        /// </summary>
        byte[] channelArray;

        /// <summary>
        /// List of points
        /// </summary>
        private PointPairList list = new PointPairList();

        /// <summary>
        /// Color used for the chart
        /// </summary>
        private Color[] colors = { Color.Red, Color.Yellow, Color.Green, Color.Blue, Color.Purple };

        /// <summary>
        /// Fill of colors, only created once
        /// </summary>
        private Fill colorsFill;

        /// <summary>
        /// Pointer that will be given to receive the data
        /// </summary>
        private IntPtr values;

        /// <summary>
        /// Array allocated once by autorefresh acquisition (spare some resources)
        /// </summary>
        private Int32[] data;

        /// <summary>
        /// Store the input count
        /// </summary>
        private byte inputCount;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public AutoRefresh()
        {
            InitializeComponent();

            // Create the useful array
            this.channelsBox = new CheckBox[16];
            this.channelsBox[0] = this.channel0;
            this.channelsBox[1] = this.channel1;
            this.channelsBox[2] = this.channel2;
            this.channelsBox[3] = this.channel3;
            this.channelsBox[4] = this.channel4;
            this.channelsBox[5] = this.channel5;
            this.channelsBox[6] = this.channel6;
            this.channelsBox[7] = this.channel7;
            this.channelsBox[8] = this.channel8;
            this.channelsBox[9] = this.channel9;
            this.channelsBox[10] = this.channel10;
            this.channelsBox[11] = this.channel11;
            this.channelsBox[12] = this.channel12;
            this.channelsBox[13] = this.channel13;
            this.channelsBox[14] = this.channel14;
            this.channelsBox[15] = this.channel15;
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AutoRefresh_Load(object sender, EventArgs e)
        {
            // Open the first board of the system
            this.board = PCI3116_NET.PCI3116.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Initialize the chart graphics
            this.InitializeChartGraphics();

            // Get the number of inputs and disable the not used one
            {
                int returnValue = 0;
                if ( ( returnValue = this.board.GetAnalogInputChannelsNbr(ref this.inputCount)) != 0)
                {
                    MessageBox.Show("GetAnalogInputChannelsNbr error : " + returnValue.ToString());
                    this.Close();
                    return;
                }

                // Deactivate not possible inputs
                for (byte i = this.inputCount; i < 16; i++)
                {
                    this.channelsBox[i].Enabled = false;
                }
            }
        }

        /// <summary>
        /// Initialize the chart graphics
        /// </summary>
        private void InitializeChartGraphics()
        {
            this.chartControl.GraphPane.Title.Text = "";
            this.chartControl.GraphPane.XAxis.MajorTic.IsOpposite = false;
            this.chartControl.GraphPane.XAxis.MinorTic.IsOpposite = false;
            this.chartControl.GraphPane.YAxis.MajorTic.IsOpposite = false;
            this.chartControl.GraphPane.YAxis.MinorTic.IsOpposite = false;

            this.chartControl.GraphPane.YAxis.Scale.MinAuto = true;
            this.chartControl.GraphPane.YAxis.Scale.MaxAuto = true;
            this.chartControl.GraphPane.YAxis.Scale.MinorStep = 1;

            this.chartControl.GraphPane.XAxis.Title.Text = "Channel";
            this.chartControl.GraphPane.YAxis.Title.Text = "Value (V)";
        }

        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AutoRefresh_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Stop the sequence if any is running
            if (startButton.Enabled == false)
            {
                this.board.StopAnalogAcquisition();
            }
        }

        /// <summary>
        /// Click on select / deselect check - box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void selectDeselectBox_CheckedChanged(object sender, EventArgs e)
        {
            // Select - deselect all
            for (int i = 0; i < this.inputCount; i++)
            {
                this.channelsBox[i].Checked = this.selectDeselectBox.Checked;
            }
        }

        /// <summary>
        /// Return the number of selected channels
        /// </summary>
        /// <returns></returns>
        private uint getSelectChannelsCount()
        {
            uint count = 0;
            for (int i = 0; i < channelsBox.Length; i++)
            {
                if (channelsBox[i].Checked)
                {
                    count++;
                }
            }
            return count;
        }

        /// <summary>
        /// Check and return the correct value of the converting time parameter
        /// </summary>
        /// <param name="convertingTime"></param>
        /// <returns></returns>
        private bool checkAndGetConvertingTime(ref uint convertingTime)
        {
            try
            {
                convertingTime = Convert.ToUInt32(this.convertingTimeText.Text);
                if ( (convertingTime < 5) || (convertingTime > 65535000 ) )
                {
                    convertingTime = 0;
                    return false;
                }
                return true;
            }
            catch (Exception)
            {
                convertingTime = 0;
                return false;
            }
        }

        /// <summary>
        /// Construct the channel array
        /// </summary>
        /// <returns></returns>
        private byte[] getChannelArray()
        {
            int currentIndex = 0;
            byte[] array = new byte[getSelectChannelsCount()];
            for (int i = 0; i < this.inputCount; i++)
            {
                if (this.channelsBox[i].Checked == true)
                {
                    array[currentIndex] = (byte)i;
                    currentIndex++;
                }
            }
            return array;
        }

        /// <summary>
        /// Click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            uint convertingTime = 0;
            byte[] gainArray;
            byte[] polarityArray;
            int returnValue = 0;
            uint channelCount = getSelectChannelsCount();

            if (channelCount == 0)
            {
                MessageBox.Show("Select at least one channel");
                return;
            }

            // Check the analog input mode
            if ((seRadio.Checked == false) && (diffRadio.Checked == false))
            {
                MessageBox.Show("Please choose the input mode");
                return;
            }

            if (this.checkAndGetConvertingTime(ref convertingTime) == false)
            {
                MessageBox.Show("Converting time : invalid value");
                return;
            }

            // Construct the gain and polarity array
            gainArray = new byte[this.inputCount];
            polarityArray = new byte[this.inputCount];
            for (int i = 0; i < this.inputCount; i++)
            {
                gainArray[i] = 0; /* gain 1*/
                polarityArray[i] = 1; /* bipolar */
            }

            // Construct the channel array
            this.channelArray = getChannelArray();

            // Create the data array
            this.data = new Int32[this.channelArray.Length];

            // Start the chart now that we have the whole configuration
            this.startChart();

            // Start the autorefresh acquisition
            returnValue = this.board.InitAndStartAutoRefreshAcquisition((byte)channelCount,
                channelArray,
                gainArray,
                polarityArray,
                convertingTime,
                ref values); // Values will point on the acquisition's data

            if (returnValue != 0)
            {
                MessageBox.Show("Error starting acquisition : " + returnValue.ToString());
                return;
            }

            // Update the graphics
            this.startButton.Enabled = false;
            
            this.displayTimer.Start();
        }

        /// <summary>
        /// Start the chart
        /// </summary>
        private void startChart()
        {
            BarItem curve;
            String[] channelTitle = new String[this.channelArray.Length];

            this.chartControl.GraphPane.CurveList.Clear();
            this.list.Clear();

            for (int i = 0; i < channelArray.Length; i++)
            {
                channelTitle[i] = "Ch. " + channelArray[i].ToString();
                list.Add(i, 0, i / 4.0);
            }

            // Add the curve
            curve = this.chartControl.GraphPane.AddBar("Values", list, Color.Blue);

            // Create the color handling
            colorsFill = new Fill(colors);
            curve.Bar.Fill = colorsFill;
            curve.Bar.Fill.Type = FillType.GradientByZ;
            curve.Bar.Fill.RangeMin = 0;
            curve.Bar.Fill.RangeMax = 4;

            // Set the title line
            this.chartControl.GraphPane.XAxis.Scale.TextLabels = channelTitle;
            this.chartControl.GraphPane.XAxis.Type = AxisType.Text;
            this.chartControl.AxisChange();
            this.chartControl.Invalidate();
        }

        /// <summary>
        /// SE mode is set
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void seRadio_CheckedChanged(object sender, EventArgs e)
        {
            if (this.seRadio.Checked)
            {
                int returnValue = this.board.SetAnalogInputMode(1);
                if (returnValue != 0)
                {
                    MessageBox.Show("SetAnalogInputMode error : " + returnValue.ToString());
                    return;
                }
            }
        }

        /// <summary>
        /// DIFF mode is set
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void diffRadio_CheckedChanged(object sender, EventArgs e)
        {
            if (this.diffRadio.Checked)
            {
                int returnValue = this.board.SetAnalogInputMode(2);
                if (returnValue != 0)
                {
                    MessageBox.Show("SetAnalogInputMode error : " + returnValue.ToString());
                    return;
                }
            }
        }

        /// <summary>
        /// Click on the stop button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            int returnValue = this.board.StopAnalogAcquisition();

            this.displayTimer.Stop();

            if (returnValue != 0)
            {
                MessageBox.Show("StopAnalogAcquisition error : " + returnValue.ToString());
                return;
            }
            this.startButton.Enabled = true;
        }

        /// <summary>
        /// Display timer tick
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void displayTimer_Tick(object sender, EventArgs e)
        {
            BarItem curve;
            this.chartControl.GraphPane.CurveList.Clear();

            // Get the value from the pointer
            for (int i = 0; i < this.channelArray.Length; i++)
            {
                this.data[i] = Marshal.ReadInt32(this.values, i * 4);
            }

            byte resolution = 0;
            this.board.GetAnalogInputResolution(ref resolution);
            int maxVal = (int)Math.Pow(2, resolution - 1) - 1;			
			
            for (int i = 0; i < this.channelArray.Length; i++)
            {
                // Compute the voltage value
                // Gain 1, bipolar
                list[i].Y = ((double)data[i] - maxVal) * (10.0 / maxVal);
 
            }

            // Add the curve
            curve = this.chartControl.GraphPane.AddBar("Values", list, Color.Blue);

            // Create the color handling
            curve.Bar.Fill = colorsFill;
            curve.Bar.Fill.Type = FillType.GradientByZ;
            curve.Bar.Fill.RangeMin = 0;
            curve.Bar.Fill.RangeMax = 4;

            this.chartControl.AxisChange();
            this.chartControl.Invalidate();
        }
    }
}